#!/usr/bin/env node
/*!
 * gitmars v7.0.0-beta.22
 * 这是一个git工作流工具
 * (c) 2021-2024 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import dayjs from 'dayjs';
import { checkbox, select } from '@inquirer/prompts';
import sh from 'shelljs';
import chalk from 'chalk';
import to from 'await-to-done';
import { queue } from '@gitmars/core';
import { getIsGitProject, getCurrentBranch, checkGitStatus, fetch, prune, searchBranches, getGitLogsByCommitIDs, getGitLogs } from '@gitmars/git';
import { createArgs, echo } from '@gitmars/utils';
import { cmdConfig } from './conf/copy.mjs';
import lang from './common/local.mjs';

const { t } = lang;
const { blue, green, red, yellow } = chalk;
const { args, options } = cmdConfig;
if (!getIsGitProject()) {
  sh.echo(red(t("The current directory is not a git project directory")));
  process.exit(1);
}
program.name("gitm copy").usage("[commitid...] [--lastet [lastet]] [--limit [limit]] [--no-merges]").description(
  t(
    "cherry-pick batch version, copy a record from a branch and merge it into the current branch"
  )
);
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (commitid, opts) => {
  const keys = ["%H", "%T", "%P", "%aI", "%an", "%s", "%b"];
  const current = getCurrentBranch();
  const status = checkGitStatus();
  let logList = [], cmd = [];
  if (!status)
    process.exit(1);
  fetch();
  prune();
  const branches = searchBranches();
  if (commitid.length > 0) {
    logList = getGitLogsByCommitIDs({ commitIDs: commitid, keys });
  } else {
    logList = getGitLogs({
      lastet: opts.lastet,
      limit: opts.limit,
      noMerges: !opts.merges,
      keys
    });
    if (logList.length === 0) {
      echo(
        yellow(
          t(
            "No eligible commit logs found, please relax the filtering conditions appropriately. The process has been exited"
          )
        )
      );
      process.exit(0);
    }
  }
  const [, commitIDs = []] = await to(
    checkbox({
      message: t("Please select the commit record to copy"),
      choices: logList.map((log, index) => {
        const _time = dayjs(log["%aI"]).format("YYYY/MM/DD HH:mm");
        return {
          name: `${green(index + 1 + ".")} ${green(log["%s"])} | ${yellow(log["%an"])} | ${blue(
            _time
          )}`,
          value: log["%H"],
          checked: false
        };
      })
    })
  );
  if (commitIDs.length === 0) {
    echo(yellow(t("No commit record selected, process has exited")));
    process.exit(0);
  }
  const [, chooseBranch] = await to(
    select({
      message: t("Please select the target branch"),
      choices: branches.map((item) => ({ name: item, value: item }))
    })
  );
  cmd = [
    `git checkout ${chooseBranch}`,
    "git pull",
    {
      cmd: `git cherry-pick ${commitIDs.reverse().join(" ")}`,
      config: {
        again: false,
        success: t("Record merge successful"),
        fail: t("Merge failed, please follow the instructions")
      }
    },
    `git checkout ${current}`
  ];
  queue(cmd);
});
program.parse(process.argv);
