#!/usr/bin/env node
/*!
 * gitmars v7.0.0-beta.22
 * 这是一个git工作流工具
 * (c) 2021-2024 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import ora from 'ora';
import chalk from 'chalk';
import { createArgs, echo, spawnSync } from '@gitmars/utils';
import { cmdConfig } from './conf/install.mjs';
import lang from './common/local.mjs';

const { t } = lang;
const { green, red } = chalk;
const { args, options } = cmdConfig;
program.name("gitm install").usage("<pluginName> [version] [-m --mirror] [-c --client [client]] [-r --registry <registry>]").description(t("Installing plugins, e.g. @gitmars/core"));
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action((pluginName, version, opt) => {
  if (!pluginName) {
    echo(red(t("Enter the plugin name")));
    process.exit(1);
  }
  const spinner = ora();
  if (version) {
    const match = version.match(/[0-9.]+$/);
    if (match)
      version = match[0];
    else if (!["alpha", "lite", "beta", "release", "latest", "next"].includes(version)) {
      console.error(
        t(
          "Incorrect version number entered, only supported: alpha, lite, beta, release, latest, next"
        )
      );
      process.exit(0);
    }
  } else {
    version = "latest";
  }
  let cmdAdd;
  switch (opt.client) {
    case "yarn":
      cmdAdd = [opt.client, ["global", "add", `${pluginName}@${version}`]];
      break;
    case "pnpm":
      cmdAdd = [opt.client, ["add", "-g", `${pluginName}@${version}`]];
      break;
    default:
      cmdAdd = [opt.client, ["install", "-g", `${pluginName}@${version}`]];
      break;
  }
  if (!opt.registry && opt.mirror) {
    opt.registry = "https://registry.npmmirror.com";
  }
  if (opt.registry) {
    cmdAdd[1] = cmdAdd[1].concat(["-registry", opt.registry]);
  }
  spinner.start(green(t("Installing")));
  const install = spawnSync(...cmdAdd, {
    stdio: "ignore",
    shell: process.platform === "win32"
  });
  if (install.status === 0) {
    spinner.succeed(green(t("Installation complete")));
  } else {
    spinner.fail(red(t("There was an installation error, please contact the administrator")));
  }
  spinner.stop();
  process.exit(0);
});
program.parse(process.argv);
