#!/usr/bin/env node
/*!
 * gitmars v7.0.0-beta.22
 * 这是一个git工作流工具
 * (c) 2021-2024 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import sh from 'shelljs';
import chalk from 'chalk';
import { isNeedUpgrade, upgradeGitmars, queue } from '@gitmars/core';
import { getIsGitProject, getConfig, checkGitStatus, fetch, searchBranches, getCurrentBranch, getIsMergedTargetBranch } from '@gitmars/git';
import { createArgs } from '@gitmars/utils';
import { defaults } from './common/global.mjs';
import { cmdConfig } from './conf/update.mjs';
import lang from './common/local.mjs';

const { t } = lang;
const { red } = chalk;
const { args, options } = cmdConfig;
if (!getIsGitProject()) {
  sh.echo(red(t("The current directory is not a git project directory")));
  process.exit(1);
}
const config = getConfig();
program.name("gitm update").usage("[type] [name] [--use-merge] [--use-rebase] [-a --all] [-f --force]").description(
  t(
    "Update bug task branch, update feature function development branch, framework adjustment branch support"
  )
);
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (type, name, opt) => {
  const needUpgrade = await isNeedUpgrade(config.versionControlType);
  needUpgrade && upgradeGitmars();
  const allow = ["bugfix", "feature", "support"];
  const deny = [
    defaults.master,
    defaults.develop,
    defaults.release,
    defaults.bugfix,
    defaults.support
  ];
  const status = checkGitStatus();
  let cmds = [], branchList = [], _nameArr = [];
  if (!status)
    process.exit(1);
  fetch();
  if (opt.all) {
    if (!type)
      type = allow.join(",");
    branchList = searchBranches({ type });
  } else if (!type || !name) {
    const current = getCurrentBranch();
    [type, ..._nameArr] = current.split("/");
    name = _nameArr.join("/");
    if (!name) {
      deny.includes(type) && sh.echo(
        red(
          t(
            "Hey bro, what is the fuck are you doing by executing this command in the {type} branch?",
            { type }
          )
        )
      );
      process.exit(1);
    }
    if (!allow.includes(type)) {
      sh.echo(red(t("type only allows input") + ": " + JSON.stringify(allow)));
      process.exit(1);
    }
    branchList = branchList.concat(current);
  } else if (!allow.includes(type)) {
    sh.echo(red(t("type only allows input") + ": " + JSON.stringify(allow)));
    process.exit(1);
  } else {
    branchList = [type + "/" + name];
  }
  branchList.forEach((branch) => {
    [type, ..._nameArr] = branch.split("/");
    name = _nameArr.join("/");
    const base = type === "bugfix" ? config.bugfix : type === "support" ? config.master : config.release;
    const isNeedCombine = !getIsMergedTargetBranch(base, `${type}/${name}`, { remote: true });
    let cmd = [];
    if (isNeedCombine || opt.force) {
      cmd = cmd.concat([
        "git fetch",
        `git checkout ${base}`,
        "git pull",
        `git checkout ${type}/${name}`
      ]);
      if (opt.useRebase) {
        cmd.push({
          cmd: `git rebase ${base}`,
          config: {
            again: false,
            success: t("Merge {source} to {target} successfully", {
              source: base,
              target: `${type}/${name}`
            }),
            fail: t(
              "An error occurred merging {source} to {target}, please follow the instructions",
              {
                source: base,
                target: `${type}/${name}`
              }
            )
          }
        });
      } else {
        cmd.push({
          cmd: `git merge --no-ff ${base}`,
          config: {
            again: false,
            success: t("Merge {source} to {target} successfully", {
              source: base,
              target: `${type}/${name}`
            }),
            fail: t(
              "An error occurred merging {source} to {target}, please follow the instructions",
              {
                source: base,
                target: `${type}/${name}`
              }
            )
          }
        });
      }
    } else {
      cmd = [
        {
          message: t("{source} has been merged with {target}", {
            source: base,
            target: `${type}/${name}`
          })
        }
      ];
    }
    cmds = cmds.concat(cmd);
  });
  queue(cmds);
});
program.parse(process.argv);
